package jp.co.kayo.android.localplayer.adapter;

import java.io.File;
import java.util.List;

import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.util.SdCardAccessHelper;
import jp.co.kayo.android.localplayer.util.bean.EditItem;
import jp.co.kayo.android.localplayer.util.bean.EditItem.ITEM_TYPE;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewParent;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.ImageView;
import android.widget.TextView;

public class EditItemAdapter extends ArrayAdapter<EditItem> implements OnCheckedChangeListener {
    LayoutInflater inflater;
    boolean isSdCard;

    public EditItemAdapter(Context context, List<EditItem> list) {
        super(context, R.layout.tageditor_list_item, list);
        inflater = (LayoutInflater) context
                .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        isSdCard = ContentsUtils.isSDCard(context);
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        ViewHolder holder = null;
        EditItem item = getItem(position);
        if (convertView == null) {
            holder = new ViewHolder();
            convertView = inflater.inflate(R.layout.tageditor_list_item,
                    null, false);
            convertView.setTag(holder);

            holder.checkbox1 = (CheckBox) convertView
                    .findViewById(R.id.checkBox1);
            holder.text1 = (TextView) convertView.findViewById(R.id.text1);
            holder.text2 = (TextView) convertView.findViewById(R.id.text2);
            holder.text3 = (TextView) convertView.findViewById(R.id.text3);
            holder.imgCache = (ImageView) convertView
                    .findViewById(R.id.imgCache);
            if (item.type == ITEM_TYPE.ALBUM) {
                holder.setAlbumVisible();
            } else if (item.type == ITEM_TYPE.ARTIST) {
                holder.setArtistVisible();
            } else if (item.type == ITEM_TYPE.SONG) {
                holder.setMediaVisible();
            }
            
            holder.checkbox1.setOnCheckedChangeListener(this);

        } else {
            holder = (ViewHolder) convertView.getTag();
        }

        holder.position = position;
        holder.checkbox1.setChecked(item.checked);

        if (item.type == ITEM_TYPE.ALBUM) {
            holder.text1.setText(item.album);
            holder.text2.setText(item.artist);
        } else if (item.type == ITEM_TYPE.ARTIST) {
            holder.text1.setText(item.artist);
        } else if (item.type == ITEM_TYPE.SONG) {
            holder.text1.setText(item.title);
            holder.text2.setText(item.album);
            holder.text3.setText(item.artist);
            if (isSdCard != true && item.data!=null && new File(item.data).exists()) {
                holder.imgCache.setVisibility(View.VISIBLE);
            } else {
                holder.imgCache.setVisibility(View.GONE);
            }
        }

        return convertView;
    }

    class ViewHolder {
        int position;
        CheckBox checkbox1;
        TextView text1;
        TextView text2;
        TextView text3;
        ImageView imgCache;

        public void setAlbumVisible() {
            text2.setVisibility(View.VISIBLE);
            text3.setVisibility(View.GONE);
            imgCache.setVisibility(View.GONE);
        }

        public void setArtistVisible() {
            text2.setVisibility(View.GONE);
            text3.setVisibility(View.GONE);
            imgCache.setVisibility(View.GONE);
        }

        public void setMediaVisible() {
            text2.setVisibility(View.VISIBLE);
            text3.setVisibility(View.VISIBLE);
            imgCache.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton arg0, boolean arg1) {
        ViewGroup parent = (ViewGroup)arg0.getParent();
        ViewHolder holder = (ViewHolder)parent.getTag();
        if(holder!=null){
            EditItem item = getItem(holder.position);
            if(item!=null){
                item.checked = arg1;
            }
        }
    }
}