package jp.co.kayo.android.localplayer.fragment;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import jp.co.kayo.android.localplayer.BaseActivity;
import jp.co.kayo.android.localplayer.BaseListFragment;
import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.adapter.SearchItemAdapter;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.SystemConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.core.ContentManager;
import jp.co.kayo.android.localplayer.core.ContextMenuFragment;
import jp.co.kayo.android.localplayer.core.IProgressView;
import jp.co.kayo.android.localplayer.core.ServiceBinderHolder;
import jp.co.kayo.android.localplayer.menu.MultipleChoiceMediaContentActionCallback;
import jp.co.kayo.android.localplayer.service.IMediaPlayerService;
import jp.co.kayo.android.localplayer.task.AsyncAddPlaylistTask;
import jp.co.kayo.android.localplayer.util.AnimationHelper;
import jp.co.kayo.android.localplayer.util.FragmentUtils;
import jp.co.kayo.android.localplayer.util.Logger;
import jp.co.kayo.android.localplayer.util.MyPreferenceManager;
import jp.co.kayo.android.localplayer.util.ViewCache;
import jp.co.kayo.android.localplayer.util.bean.FavoriteItemLoader;
import jp.co.kayo.android.localplayer.util.bean.SearchItem;
import jp.co.kayo.android.localplayer.util.bean.PlaylistInfoLoader.CallType;
import jp.co.kayo.android.localplayer.util.bean.SearchItem.ITEM_TYPE;
import android.content.Context;
import android.database.Cursor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.RemoteException;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.app.LoaderManager.LoaderCallbacks;
import android.support.v4.content.Loader;
import android.view.ActionMode;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.ListView;
import android.widget.AbsListView.OnScrollListener;

public class FavoritelistFragment extends BaseListFragment implements
        ContentManager, ContextMenuFragment, IProgressView,
        LoaderCallbacks<List<SearchItem>>, OnScrollListener {
    private static final String KEY_FAVORITE = "key.favaorite";
    private ViewCache mViewCache;
    private int mFavorite;
    private MyPreferenceManager mPref;
    private ListView mListView;
    private SearchItemAdapter mAdapter;
    private Runnable mTask = null;
    private int mLoaderId = -1;
    private AnActionModeOfEpicProportions mActionMode;

    private ViewCache getCache() {
        if (mViewCache == null) {
            mViewCache = (ViewCache) getFragmentManager().findFragmentByTag(
                    SystemConsts.TAG_CACHE);
        }
        return mViewCache;
    }

    private IMediaPlayerService getBinder() {
        if (getActivity() instanceof ServiceBinderHolder) {
            return ((ServiceBinderHolder) getActivity()).getBinder();
        } else {
            return null;
        }
    }

    public static FavoritelistFragment createFragment(int favorite, Bundle args) {
        FavoritelistFragment f = new FavoritelistFragment();
        args.putInt(KEY_FAVORITE, favorite);
        f.setArguments(args);
        return f;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (mPref == null) {
            mPref = new MyPreferenceManager(getActivity());
        }

        Bundle args = getArguments();
        if (args != null) {
            mFavorite = args.getInt(KEY_FAVORITE, 5);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState) {
        View root = inflater.inflate(R.layout.search_list_view, null);

        mListView = (ListView) root.findViewById(android.R.id.list);
        mListView.setOnScrollListener(this);

        mActionMode = new AnActionModeOfEpicProportions(getActivity(),
                mListView, mHandler);

        return root;
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);

        getLoaderManager().initLoader(getFragmentId(), null, this);
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        getLoaderManager().destroyLoader(getFragmentId());
        mLoaderId = -1;
    }

    @Override
    public Loader<List<SearchItem>> onCreateLoader(int id, Bundle arg1) {
        return new FavoriteItemLoader(getActivity(), getCache(), mFavorite);
    }

    @Override
    public void onLoadFinished(Loader<List<SearchItem>> loader,
            List<SearchItem> data) {
        Logger.d("検索結果:" + data.size());
        mAdapter = new SearchItemAdapter(getActivity(), data, getCache());
        setListAdapter(mAdapter);
        mListView.setAdapter(mAdapter);
        mAdapter.notifyDataSetChanged();
    }

    @Override
    public void onLoaderReset(Loader<List<SearchItem>> arg0) {
        mAdapter = new SearchItemAdapter(getActivity(),
                new ArrayList<SearchItem>(), getCache());
        setListAdapter(mAdapter);
        mListView.setAdapter(mAdapter);
        mAdapter.notifyDataSetChanged();
    }

    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        if (mActionMode.hasMenu()) {
            mActionMode.onItemClick(l, v, position, id);
        } else {
            SearchItem item = mAdapter.getItem(position);
            if (item.type != ITEM_TYPE.GROUP) {
                // MEDIA
                if (item.type == ITEM_TYPE.SONG) {
                    final BaseActivity base = (BaseActivity) getActivity();
                    final IMediaPlayerService binder = base.getBinder();
                    final List<SearchItem> items = new ArrayList<SearchItem>();
                    items.add(item);
                    AsyncTask<Void, Void, Void> animTask = new AsyncTask<Void, Void, Void>() {
                        @Override
                        protected Void doInBackground(Void... params) {
                            playMedia(base, binder, items);
                            return null;
                        }
                    };
                    animTask.execute();
                    mPref.setResumeReloadFlag(true);
                } else if (item.type == ITEM_TYPE.ALBUM) {
                    String album_key = item.albumKey;
                    String album_name = item.album;

                    FragmentTransaction t = getFragmentManager()
                            .beginTransaction();
                    AnimationHelper.setFragmentToPlayBack(mPref, t);
                    t.replace(getId(), AlbumSongsFragment.createFragment(
                            album_key, album_name,
                            FragmentUtils.cloneBundle(this)));
                    t.addToBackStack(SystemConsts.TAG_SUBFRAGMENT);
                    t.hide(this);
                    t.commit();
                } else if (item.type == ITEM_TYPE.ARTIST) {
                    String artist_key = item.artistKey;
                    String artist = item.artist;
                    int max = -1;

                    FragmentTransaction t = getFragmentManager()
                            .beginTransaction();
                    AnimationHelper.setFragmentToPlayBack(mPref, t);
                    t.replace(
                            getId(),
                            jp.co.kayo.android.localplayer.fragment.ArtistListFragment
                                    .createFragment(artist_key, artist, max,
                                            FragmentUtils.cloneBundle(this)));
                    t.addToBackStack(SystemConsts.TAG_SUBFRAGMENT);
                    t.hide(this);
                    t.commit();
                }
            }
        }
    }

    public boolean playMedia(BaseActivity base, IMediaPlayerService binder,
            List<SearchItem> selectedItems) {
        int position = 0;
        try {
            binder.lockUpdateToPlay();
            position = binder.getPosition();
            if (position >= 0) {
                binder.clearcut();
                position++;
            } else {
                binder.clear();
                position = 0;
            }
            for (SearchItem item : selectedItems) {
                if (item.type == ITEM_TYPE.SONG) {
                    binder.addMedia(item.id, item.data);
                } else if (item.type == ITEM_TYPE.ALBUM) {
                    binder.addMediaQuery(
                            MediaConsts.MEDIA_CONTENT_URI.toString(),
                            AudioMedia._ID, AudioMedia.DATA,
                            AudioMedia.ALBUM_KEY + " = ?",
                            new String[] { item.albumKey }, AudioMedia.TRACK
                                    + "," + AudioMedia.DATA);
                } else if (item.type == ITEM_TYPE.ARTIST) {
                    binder.addMediaQuery(
                            MediaConsts.MEDIA_CONTENT_URI.toString(),
                            AudioMedia._ID, AudioMedia.DATA,
                            MediaConsts.AudioMedia.ARTIST_KEY + " = ?",
                            new String[] { item.artistKey }, AudioMedia.YEAR
                                    + "," + AudioMedia.ALBUM + ","
                                    + AudioMedia.TRACK);
                }
            }
            return true;
        } catch (RemoteException e) {
        } finally {
            try {
                binder.setPosition(position);
                binder.play();
            } catch (RemoteException e) {
            }
        }
        return false;
    }

    @Override
    public void onScroll(AbsListView arg0, int arg1, int arg2, int arg3) {
    }

    @Override
    public void onScrollStateChanged(AbsListView arg0, int scrollState) {
        Logger.d("onScrollStateChanged:" + scrollState);
        long time = mPref.getHideTime();
        if (time > 0) {
            if (scrollState == 1) {
                if (mTask != null) {
                    mHandler.removeCallbacks(mTask);
                    mTask = null;
                }
                if (getFragmentManager() != null) {
                    ControlFragment control = (ControlFragment) getFragmentManager()
                            .findFragmentByTag(SystemConsts.TAG_CONTROL);
                    if (control != null) {
                        control.hideControl(false);
                    }
                }
            } else if (scrollState == 0) {
                if (mTask != null) {
                    mHandler.removeCallbacks(mTask);
                    mTask = null;
                }
                mTask = new Runnable() {
                    @Override
                    public void run() {
                        try {
                            if (getFragmentManager() != null) {
                                ControlFragment control = (ControlFragment) getFragmentManager()
                                        .findFragmentByTag(
                                                SystemConsts.TAG_CONTROL);
                                if (control != null) {
                                    control.showControl(false);
                                }
                            }
                        } finally {
                            mTask = null;
                        }
                    }
                };
                mHandler.postDelayed(mTask, time);
            }
        }
    }

    @Override
    public void startProgress(final long max) {
        if (mAdapter != null) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    getCache().startProgress(max);
                    IMediaPlayerService binder = getBinder();
                    if (binder != null) {
                        try {
                            getCache().setPrefetchId(binder.getPrefetchId());
                        } catch (RemoteException e) {
                        }
                    }
                    mAdapter.notifyDataSetChanged();
                }
            });
        }
    }

    @Override
    public void stopProgress() {
        if (mAdapter != null) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    getCache().stopProgress();
                    mAdapter.notifyDataSetChanged();
                }
            });
        }
    }

    @Override
    public void progress(final long pos, final long max) {
        if (mAdapter != null) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    getCache().progress(pos, max);
                    mAdapter.notifyDataSetChanged();
                }
            });
        }
    }

    @Override
    public boolean onBackPressed() {
        if (mActionMode != null && mActionMode.cancelActionMode()) {
            return true;
        }
        return false;
    }

    @Override
    public void hideMenu() {
        if (mActionMode != null) {
            mActionMode.cancelActionMode();
        }
    }

    @Override
    public String selectSort() {
        getFragmentManager().popBackStack();
        return null;
    }

    @Override
    public int getFragmentId() {
        if (mLoaderId == -1) {
            mLoaderId = new Random(System.currentTimeMillis())
                    .nextInt(99999999);
        }
        return mLoaderId;
    }

    @Override
    public String getName(Context context) {
        return context.getString(R.string.lb_tab_favorite_name) + mFavorite;
    }

    @Override
    public void reload() {
        getLoaderManager().restartLoader(getFragmentId(), null, this);
    }

    @Override
    public void release() {
        // TODO Auto-generated method stub

    }

    @Override
    public void changedMedia() {
        // TODO Auto-generated method stub

    }

    @Override
    protected void datasetChanged() {
        if (mAdapter != null) {
            mAdapter.notifyDataSetChanged();
        }
    }

    @Override
    protected void addRow(Object[] values) {
        // TODO Auto-generated method stub

    }

    private final class AnActionModeOfEpicProportions extends
            MultipleChoiceMediaContentActionCallback {

        public AnActionModeOfEpicProportions(Context context,
                ListView listView, Handler handler) {
            super(context, listView, handler);
        }

        @Override
        public boolean onCreateActionMode(ActionMode mode, Menu menu) {
            menu.add(getString(R.string.sub_mnu_order))
                    .setIcon(R.drawable.ic_menu_btn_add)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);

            menu.add(getString(R.string.sub_mnu_selectall))
                    .setIcon(R.drawable.ic_menu_selectall)
                    .setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
            return true;
        }
    };

    @Override
    protected void messageHandle(int what, List<Integer> items) {
        if (items.size() > 0) {
            switch (what) {
            case SystemConsts.EVT_SELECT_ADD: {
                final BaseActivity base = (BaseActivity) getActivity();
                ArrayList<Long> ids = new ArrayList<Long>();
                for (Integer i : items) {
                    SearchItem item = mAdapter.getItem(i);
                    if (item.type == ITEM_TYPE.SONG) {
                        ids.add(item.id);
                    } else if (item.type == ITEM_TYPE.ALBUM) {
                        Cursor cursor = null;
                        try {
                            cursor = base.getContentResolver().query(
                                    MediaConsts.MEDIA_CONTENT_URI,
                                    new String[] { AudioMedia._ID },
                                    AudioMedia.ALBUM_KEY + " = ?",
                                    new String[] { item.albumKey },
                                    AudioMedia.TRACK + "," + AudioMedia.DATA);
                            if (cursor != null && cursor.moveToFirst()) {
                                do {
                                    long id = cursor.getLong(cursor
                                            .getColumnIndex(AudioMedia._ID));
                                    ids.add(id);
                                } while (cursor.moveToNext());
                            }
                        } finally {
                            if (cursor != null) {
                                cursor.close();
                            }
                        }
                    } else if (item.type == ITEM_TYPE.ARTIST) {
                        Cursor cursor = null;
                        try {
                            cursor = base.getContentResolver().query(
                                    MediaConsts.MEDIA_CONTENT_URI,
                                    new String[] { AudioMedia._ID },
                                    AudioMedia.ARTIST_KEY + " = ?",
                                    new String[] { item.artistKey },
                                    AudioMedia.YEAR + "," + AudioMedia.ALBUM
                                            + "," + AudioMedia.TRACK);
                            if (cursor != null && cursor.moveToFirst()) {
                                do {
                                    long id = cursor.getLong(cursor
                                            .getColumnIndex(AudioMedia._ID));
                                    ids.add(id);
                                } while (cursor.moveToNext());
                            }
                        } finally {
                            if (cursor != null) {
                                cursor.close();
                            }
                        }
                    }
                }

                if (ids.size() > 0) {
                    AsyncAddPlaylistTask task = new AsyncAddPlaylistTask(base,
                            getFragmentManager(), ids, CallType.TYPE_ADDDIALOG);
                    task.execute();
                }
            }
                break;
            case SystemConsts.EVT_SELECT_CHECKALL: {
                if (mActionMode.hasChoice) {
                    for (int i = 0; i < mListView.getCount(); i++) {
                        SearchItem item = mAdapter.getItem(i);
                        if (item.type != ITEM_TYPE.GROUP) {
                            mListView.setItemChecked(i, true);
                        }
                    }
                }
            }
                break;
            }
        }
    }

}
